
__version__ = "0.1.0"
__all__ = [
    "SqlContext",
    "SqlRewriter",
    "SchemaError",
    "SchemaParseException",
    "parse_flatql",
    "parse_from_hocon_path",
    "parse_from_hocon_file"
]

from typing import Optional, List

from schema import SchemaError

from .rewriter.sql_generic import SqlContext
from .schemas import View, SchemaParseException, Schema
from .parser.flatql_parser import parse_flatql
from .rewriter.sql_rewriter import SqlRewriter


def parse_from_hocon_path(path: str, suffix: Optional[str] = ".conf") -> Schema:
    from collections import defaultdict
    from .schemas import _get_files_by_suffix, schema_definition

    views, merged_views, view_dict = list(), list(), defaultdict(list)
    hocon_files = _get_files_by_suffix(path, suffix)

    for hocon_file in hocon_files:
        views = views + parse_from_hocon_file(hocon_file)
    for view in views:
        view_dict[view.name].append(view)

    for name, raw_views in view_dict.items():
        tables = []
        for raw_view in raw_views:
            tables = tables + raw_view.tables
        merged_views.append(View(name, tables))

    return Schema(merged_views)


def parse_from_hocon_file(hocon_file: str) -> List[View]:
    from pyhocon import ConfigFactory
    from .schemas import _get_files_by_suffix, _parse_views, schema_definition

    raw_views = ConfigFactory.parse_file(hocon_file)
    if len(raw_views) > 0:
        schema_definition.validate(raw_views)
        return _parse_views(raw_views)
    else:
        return []
