from collections import deque
from typing import Optional, List, Set, Dict, Union, Any


def bfs(children, start):
    parent = {}
    q = deque([start])
    while q:
        v = q.popleft()
        for w in children[v]:
            if w not in parent:
                parent[w] = v
                q.append(w)
    return parent


def detect_involved_tables(main_table: str, tables_name: List[Union[Any]], involved_tables: Set[str],
                           relation_graph: Dict[str, str], join_order: Optional[List[str]] = None):
    parent_tree = bfs(relation_graph, main_table)

    def add_involved_table(table_name):
        if table_name in involved_tables or table_name is None:
            return
        parent = parent_tree.get(table_name, None)
        add_involved_table(parent)
        involved_tables.add(table_name)
        if join_order is not None:
            join_order.append(table_name)

    for t in tables_name:
        if t != main_table:
            add_involved_table(t)
    return parent_tree, involved_tables


def decide_main_table(tables_name: List[Any], relation_graph: Dict[str, str]) -> str:
    seen, relation_edge, relation_tree_part = set(), [], {}
    for table_name in tables_name:
        relation_tree_part[table_name] = relation_graph[table_name]

    for source, target in relation_tree_part.items():
        for t in target:
            relation_edge.append((source, t))

    for left, right in relation_edge:
        if left in seen:
            return left
        if right in seen:
            return right
        seen.add(left)
        seen.add(right)
